part of TeamSpeak;

class User extends TeamSpeakEntity {
  final int database_id;
  final int clid;
  
  bool is_online = true;
  int get cid => _cid; int _cid;
  String get unique_identifier => _unique_identifier; String _unique_identifier; 
  int get idle_time => _idle_time; int _idle_time;
  String get nickname => _nickname; String _nickname;
  String get version => _version; String _version;
  String get platform => _platform; String _platform;
  bool get input_muted => _input_muted; bool _input_muted;
  bool get output_muted => _output_muted; bool _output_muted;
  Channel get default_channel => _default_channel; Channel _default_channel;
  bool get is_recording => _is_recording; bool _is_recording;
  DateTime get created => _created; DateTime _created;
  DateTime get lastconnected => _lastconnected; DateTime _lastconnected;
  int get totalconnections => _totalconnections; int _totalconnections;
  bool get away => _away; bool _away;
  String away_message; String _away_message;
  String ip; String _ip;
  bool get is_serverquery_client => _type > 0; int _type;
  int get packets_sent_total => _packets_sent_total; int _packets_sent_total;
  int get packets_received_total => _packets_received_total; int _packets_received_total;
  int get bandwidth_sent_last_second_total => _bandwidth_sent_last_second_total; int _bandwidth_sent_last_second_total;
  int get bandwidth_received_last_second_total => _bandwidth_received_last_second_total; int _bandwidth_received_last_second_total;
  
  Channel get channel => _channel; Channel _channel;
  
  Future<User> update(){
    return _client.send(new Command("clientinfo", parameters: {"clid": clid}))
        .then((Answer answer){
          final Map<String, String> info = answer.items[0];
          _unique_identifier = info["client_unique_identifier"];
          _idle_time = int.parse(info["client_idle_time"]);
          _cid = int.parse(info["cid"]);
          _nickname = info["client_nickname"];
          if(!server.channels.containsKey(_cid)){
             throw new StateError("${_nickname} is in a channel that does not exist on the server-mirror.");
          }
          _version = info["client_version"];
          _platform = info["client_platform"];
          _input_muted = info["client_input_muted"] == "1";
          _output_muted = info["client_output_muted"] == "1";
          _is_recording = info["client_is_recording"] == "1";
          _created = new DateTime.fromMillisecondsSinceEpoch(int.parse(info["client_created"])*1000);
          _lastconnected = new DateTime.fromMillisecondsSinceEpoch(int.parse(info["client_lastconnected"])*1000);
          _totalconnections = int.parse(info["client_totalconnections"]);
          _away = info["client_away"] == "1";
          _away_message = info["client_away_message"];
         _ip = info["connection_client_ip"];
         _type = int.parse(info["client_type"]);
         _packets_sent_total = int.parse(info["connection_packets_sent_total"]);
         _packets_received_total = int.parse(info["connection_packets_received_total"]);
         _bandwidth_received_last_second_total = int.parse(info["connection_bandwidth_received_last_second_total"]);
         _bandwidth_sent_last_second_total = int.parse(info["connection_bandwidth_sent_last_second_total"]);         
          
          try {
            final int default_channel_id = int.parse(info["client_default_channel"].substring(1));
            _default_channel = (server.channels.containsKey(default_channel_id) ? server.channels[default_channel_id] : null);
          } catch(e){
            _default_channel = null;
          }
          
          _channel = server.channels[_cid];
          
          return this;
        }, onError: (Error e){
          is_online = false;
        });
  }
  
  User(Server server, _Client _client, int this.clid, int this.database_id) 
    : super(server, _client);
}